import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import ChartDisplay;
import BasicWindowMonitor;


/******************************************/
/*                                        */
/* Class: Charge                          */
/*                                        */
/******************************************/

public class Charge extends JFrame {

  public boolean isChanged;

  private Chart chart;
  private JScrollPane scrollPane;
  private JButton forwardButton, backwardButton;
  private IndexField indexField;



  /******************************************/
  /*                                        */
  /* Constructor: Charge                    */
  /*                                        */
  /******************************************/

  public Charge( String[] args )
  {
    super("charge");
    try {
      chart = new Chart( args );
    }
    catch (FileNotFoundException e) {
      System.err.println("Error: Can't open input file");
      dispose();
      System.exit(0);
    }
    catch (IOException e) {
      System.err.println("Error: while reading from input file");
      dispose();
      System.exit(0);
    }
  }


  /******************************************/
  /*                                        */
  /* Method: initDisplay                    */
  /*                                        */
  /******************************************/

  private void initDisplay()
  {
    try {
      UIManager.setLookAndFeel("com.sun.java.swing.plaf.metal.MetalLookAndFeel");
      SwingUtilities.updateComponentTreeUI(this);
    }
    catch (ClassNotFoundException e) {}
    catch (UnsupportedLookAndFeelException e) {}
    catch (InstantiationException e) {}
    catch (IllegalAccessException e) {}

    getContentPane().add(new ChargeBox(), BorderLayout.NORTH);
    addWindowListener(new BasicWindowMonitor(false));
  }


  /******************************************/
  /*                                        */
  /* Method: newDisplay                     */
  /*                                        */
  /******************************************/

  public void newDisplay ( int i ) throws IndexOutOfBoundsException
  {
    JPanel display;

    if (scrollPane != null) {
      getContentPane().remove(scrollPane);
    }

    try {
      chart.select(i);
    }
    catch (IOException e) {
      System.err.println("Error: while reading from input");
      dispose();
      System.exit(0);
    }

    if (chart.selectedParseForest().unparsed) {
      JLabel label = new JLabel("Parsing failed!");
      display = new JPanel();
      display.add(label);
    }
    else {
      display = (JPanel) new ChartDisplay( chart );
    }

    scrollPane = new JScrollPane(display);
    getContentPane().add(scrollPane, BorderLayout.CENTER);

    backwardButton.setEnabled((i==0) ? false : true);
    forwardButton.setEnabled((i==chart.currentSize()-1) ? false : true);

    indexField.setText(Integer.toString(i+1));
    pack();
  }


  /******************************************/
  /*                                        */
  /* Subclass: ChargeBox                    */
  /*                                        */
  /******************************************/

  private class ChargeBox extends Box {

    public ChargeBox() {
      super(BoxLayout.X_AXIS);

      backwardButton = new JButton("<");
      backwardButton.setToolTipText("show previous sentence");
      backwardButton.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  newDisplay(chart.currentIndex()-1);
	}
      });
      backwardButton.setMnemonic(KeyEvent.VK_B);
      add(backwardButton);    

      forwardButton = new JButton(">");
      forwardButton.setToolTipText("show next sentence");
      forwardButton.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  newDisplay(chart.currentIndex()+1);
	}
      });
      forwardButton.setMnemonic(KeyEvent.VK_F);
      add(forwardButton);    

      add(new JLabel("  sentence "));
      indexField = new IndexField();
      add(indexField);

      add(Box.createHorizontalGlue());

      JButton button = new JButton("Quit");
      button.setToolTipText("exit Charge");
      button.setMnemonic(KeyEvent.VK_Q);
      button.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent e) {
	  dispose();
	  System.exit(0);
	}
      });
      add(button);
    }
  }

  /******************************************/
  /*                                        */
  /* Subclass: IndexField                   */
  /*                                        */
  /******************************************/

  private class IndexField extends JTextField implements ActionListener {

    public IndexField() {
      super(5);
      setHorizontalAlignment(RIGHT);
      setMaximumSize(new Dimension(50,20));
      addActionListener(this);
    }

    public void actionPerformed(ActionEvent e) {
      String s = getText();
      int i;
      try {
	i = Integer.parseInt(s);
      }
      catch (NumberFormatException nfe) {
	JOptionPane.showMessageDialog(scrollPane, "\""+s+"\" is not a number",
				      "Error", JOptionPane.WARNING_MESSAGE);
	return;
      }
      try {
	newDisplay(i-1);
      }
      catch (IndexOutOfBoundsException iob) {
	JOptionPane.showMessageDialog(scrollPane,
				      "Number "+i+" is out of range",
				      "Error", JOptionPane.WARNING_MESSAGE);
	return;
      }
    }
  }
  
  /******************************************/
  /*                                        */
  /* Method: main                           */
  /*                                        */
  /******************************************/

  public static void main(String[] args)
  {
    Charge c = new Charge( args );
    c.initDisplay();
    try {
      c.newDisplay(0);
    }
    catch (IndexOutOfBoundsException e) {
      System.err.println("Error: input is empty");
      c.dispose();
      System.exit(0);
    }
    c.setVisible(true);
  }
}
